require('mm_expand');
const exceljs = require('exceljs');

/**
 * 执行Excel
 */
class Excel {
	/**
	 * 构造函数
	 * @param {Object} config 配置参数
	 */
	constructor(config) {
		var time = new Date();
		this.config = {
			path: __dirname,
			file: './number.xlsx',
			convert: true,
			// extensions: "xlsx",
			sheet: 1, // 'Sheet1'
			excel: {
				// 创建者
				creator: 'MM',
				// 最后修改者
				editor: 'MM',
				// 创建时间
				create_time: time,
				// 修改时间
				edit_time: time,
				// 最后修改时间
				last_time: time
			},
			pageSetup: {
				orientation: 'landscape', // 设置纸张方向为纵向（'portrait'）或横向（'landscape'）  
				fitToPage: true, // 是否将内容调整为适合页面  
				fitWidth: 1, // 宽度缩放比例
				fitHeight: 1, // 高度缩放比例
				// 页边距 1cm = 2.54
				margins: {
					left: 0.25, // 左侧页边距
					right: 0.25, // 右侧页边距
					top: 0.75, // 上部页边距
					bottom: 0.75, // 下部页边距  
					header: 0.3, // 页眉边距
					footer: 0.3, // 页脚边距
				}
				// ... 其他页面设置  
			},
			cell_style: {
				alignment: {
					vertical: 'middle',
					horizontal: 'center'
				},
				border: {
					top: {
						style: 'thin',
						color: {
							argb: 'FF808080'
						}
					},
					bottom: {
						style: 'thin',
						color: {
							argb: 'FF808080'
						}
					},
					left: {
						style: 'thin',
						color: {
							argb: 'FF808080'
						}
					},
					right: {
						style: 'thin',
						color: {
							argb: 'FF808080'
						}
					}
				}
			},
			format: [],
			params: [],
			tpl: {
				head: "",
				body: "",
				main: ""
			},
			option: null
		};
		// 每一列的样式
		this.row_style = null;
		// 每一行的高度
		this.row_height = 0;

		this.init(config);
	}
}

/**
 * 初始化excel配置
 * @param {Object} config
 */
Excel.prototype.init = function(config) {
	$.push(this.config, config, true);
	// 原始值
	this.original = [];

	// 转换后的值
	this.list = [];

	// excel工作簿
	this.book = null;

	// 单表窗
	this.sheet = null;
	this.add_book();
}

/**
 * 新建参数模型
 * @param {Object} key
 * @param {Array} arr
 * @return {Object} 返回格式
 */
Excel.prototype.model = function(name, title, type) {
	return {
		name,
		title,
		type
	};
};

/**
 * 新建格式
 * @param {Array} list 列表
 * @param {String} id ID字段
 * @param {String} name 名称字段
 * @param {String} title 标题字段
 * @return {Object} 返回格式模型
 */
Excel.prototype.format = function(list, id, name, title) {
	return {
		list,
		id,
		name,
		title
	};
};

/**
 * 创建表
 * @param {Object} config 配置参数
 */
Excel.prototype.add_book = function(config) {
	if (!config) {
		config = this.config.excel;
	}
	var book = new exceljs.Workbook();
	book.creator = config.creator;
	book.lastModifiedBy = config.editor;
	book.created = config.create_time;
	book.modified = config.edit_time;
	book.lastPrinted = config.last_time;
	this.book = book;
};

/**
 * 读取过滤取值
 * @param {Object} item 对象
 * @return {Object} 返回想要的对象值
 */
Excel.prototype.load_filter = function(item) {
	var fmt = this.config.format;
	fmt.map((m) => {
		var lt = m.list;
		if (!m.id) {
			m.id = m.key;
		}
		if (m.table) {
			for (var i = 0; i < lt.length; i++) {
				var o = lt[i];
				if (o[m.name] == item[m.key]) {
					item[m.key] = o[m.id];
				}
			}
		} else {
			for (var i = 0; i < lt.length; i++) {
				var val = lt[i];
				if (val == item[m.key]) {
					item[m.key] = i;
				}
			}
		}
	});
	return item;
};

/**
 * 读取对应列
 */
Excel.prototype.load_col = function(sheet, n = 1) {
	var col = this.columns();
	if (!sheet) {
		sheet = this.sheet;
	}
	var row = sheet.getRow(n);
	var kv = {};
	var idx = 0;
	var fmt = this.config.format;
	row._cells.forEach((o) => {
		idx++;
		for (var i = 0; i < col.length; i++) {
			var oj = col[i];
			if (o.value == oj.header) {
				kv[idx] = {
					key: oj.key,
					dataType: oj.type || 'varchar'
				};
				continue;
			};
		}
		for (var i = 0; i < fmt.length; i++) {
			var oj = fmt[i];
			if (o.value == oj.title) {
				kv[idx] = {
					key: oj.key,
					dataType: oj.type || 'int'
				};
				continue;
			};
		}
	});
	return kv;
};

/**
 * 保存对应列
 */
Excel.prototype.save_col = function() {
	var col = this.columns();
	var idx = 0;
	var fmt = this.config.format;
	fmt.map((m) => {
		for (var i = 0; i < col.length; i++) {
			var o = col[i];
			if (o.key == m.key) {
				o.header = m.title;
				o.dataType = 'varchar';
			}
		}
	});
	return col;
};

/**
 * 保存过滤取值
 * @param {Object} item 对象
 * @return {Object} 返回想要的对象值
 */
Excel.prototype.save_filter = function(item) {
	var jobj = Object.assign({}, item);
	var fmt = this.config.format;
	fmt.map((m) => {
		if (m.table) {
			for (var k in jobj) {
				if (k == m.key) {
					var lt = m.list;
					for (var i = 0; i < lt.length; i++) {
						var o = lt[i];
						if (o[k] == jobj[k]) {
							jobj[m.key] = o[m.name];
						}
					}
				}
			}
		} else {
			for (var k in jobj) {
				if (k == m.key) {
					if (jobj[k] !== '' && jobj[k] !== undefined && jobj[k] !== null) {
						var index = jobj[k];
						if (index < m.list.length) {
							jobj[k] = m.list[index];
						} else {
							jobj[k] = '';
						}
					} else {
						jobj[k] = '';
					}
				}
			}
		}
	});
	return jobj;
};

/**
 * 打开一份表
 * @param {Object} file
 * @param {Object} nameOrId
 */
Excel.prototype.open = async function(file, nameOrId) {
	if (!file) {
		return null;
	}
	var arr = file.split('.');
	var ext = arr[arr.length - 1];
	if (ext === 'xls') {
		ext = 'xlsx';
	}
	var f = file.fullname(this.config.path);
	if (f.hasFile()) {
		await this.book[ext].readFile(f);
		return this.book.getWorksheet(nameOrId !== undefined ? nameOrId : this.config.sheet);
	} else {
		return null;
	}
}

/**
 * 读取excel
 * @param {Object} func 加载函数
 * @param {String|Number} nameOrId 名称或表id
 * @param {boolean} toList 转为列表
 * @return {Object} 返回excel对象
 */
Excel.prototype.load = async function(func, nameOrId, toList = true) {
	if (!toList) {
		return [];
	}
	var cg = this.config;
	this.sheet = await this.open(cg.file, nameOrId);
	if (!this.sheet) {
		return [];
	}
	var sheet = this.sheet;
	var list = [];
	var cols = this.load_col(sheet);
	if (func) {
		sheet.eachRow((o) => {
			var obj = {};
			for (var k in cols) {
				var val = o.getCell(Number(k)).value;
				if (val && /\-?[0-9.]+/.test(val)) {
					if (cols[k].dataType.indexOf('int') !== -1 || cols[
							k]
						.dataType === 'double' || cols[k].dataType ===
						'float') {
						val = Number(val || '0');
					}
				}
				obj[cols[k].key] = val;
			}
			list.push(func(obj));
		});
	} else {
		sheet.eachRow((o) => {
			var obj = {};
			for (var k in cols) {
				var val = o.getCell(Number(k)).value;
				if (val && /\-?[0-9.]+/.test(val)) {
					if (cols[k].dataType.indexOf('int') !== -1 ||
						cols[
							k].dataType === 'double' || cols[k]
						.dataType ===
						'float') {
						val = Number(val || '0');
					}
				}
				obj[cols[k].key] = val;
			}
			list.push(this.load_filter(obj));
		});
	}
	var lt = list.length > 0 ? list.splice(1) : [];
	return lt;
}

/**
 * 设置列
 */
Excel.prototype.columns = function() {
	var columns = [];
	var cg = this.config;
	var lt = cg.params;
	for (var i = 0; i < lt.length; i++) {
		var o = lt[i];
		var key = o.name;
		var fmt = cg.format[key] || {};
		var m = Object.assign({
			key,
			header: o.title,
			type: o.type,
			dataType: o.dataType,
			style: {}
		}, fmt);
		if (o.dataType == 'datetime' || o.dataType == 'timestamp') {
			m.style = {
				numFmt: 'yyyy-dd-mm hh:mm:ss'
			};
		} else if (o.dataType == 'date') {
			m.style = {
				numFmt: 'yyyy-dd-mm'
			};
		} else if (o.dataType == 'time') {
			m.style = {
				numFmt: 'hh:mm:ss'
			};
		}
		m.style.alignment = {
			vertical: 'middle',
			horizontal: 'center'
		};
		columns.push(m);
	}
	return columns;
};

/**
 * 合并表格
 * @param {Object} sheet1
 * @param {Object} sheet2
 * @param {Boolean} bl 是否使用源表样式
 */
Excel.prototype.merge_sheet = function(sheet1, sheet2, bl = true) {
	if (!sheet1) {
		return sheet2;
	}
	if (!sheet2) {
		return sheet1;
	}
	// 写入第三行和第四行到目标工作表
	var count = sheet1.rowCount;
	for (var i = 1; i <= sheet2.rowCount; i++) {
		var rowSource = sheet2.getRow(i);
		var rowTarget = sheet1.addRow();
		rowTarget.height = rowSource.height;
		rowTarget.style = rowSource.style;

		rowSource.eachCell({
			includeEmpty: true
		}, (cell, colNumber) => {
			var c = rowTarget.getCell(colNumber);
			c.value = cell.value;
			if (bl) {
				c.style = cell.style; // 根据需要启用或调整，复制值和样式
			}
		});
	}
	// 更新合并的单元格信息
	var dict = sheet2._merges;
	for (var k in dict) {
		var o = dict[k];
		var startRow = count + o.top;
		var endRow = count + o.bottom;
		var startCol = o.left;
		var endCol = o.right;
		sheet1.mergeCells(startRow, startCol, endRow, endCol);
	}
	return sheet1
}

/**
 * 合并列配置
 * @param {Object} columns1
 * @param {Object} columns2
 */
Excel.prototype.merge_columns = function(columns1, columns2) {
	if (!columns2) {
		return columns1
	}
	if (!columns1) {
		return columns2
	}
	return columns1;
}

/**
 * 补全列信息
 * @param {Object} sheet
 */
Excel.prototype.full_columns = function(sheet, n) {
	var col = this.save_col();
	var row = sheet.getRow(n);
	var kv = {};
	var idx = 0;
	var fmt = this.config.format;
	var list = row._cells;
	for (var i = 0; i < list.length; i++) {
		var o = list[i];
		var val = o.value;
		if (val) {
			var header;
			if (typeof(val) == 'number') {
				header = val.toString();
			} else if (typeof(val) == 'object') {
				header = val.richText.map((o) => {
					return o.text
				}).join('');
			} else {
				header = val;
			}
			var obj = col.getObj({
				header
			});
			if (obj) {
				delete obj.header;
				Object.assign(sheet.columns[i], obj);
			}
		}
	}
	return sheet.columns;
}

/**
 * @param {Object} sheet
 * @param {Object} sheet_new
 */
Excel.prototype.copy_sheet = function(sheet, sheet_new) {
	Object.assign(sheet, sheet_new);
}

/**
 * 追加sheet到book中
 * @param {Object} sheet
 */
Excel.prototype.add_sheet = function(sheet) {
	var st = this.book.worksheets.addWorksheet(sheet.name);
	Object.assign(st, sheet);
}

/**
 * 用于处理特殊样式
 * @param {Object} cell 单元格
 */
Excel.prototype.set_cell_func = function(cell, style) {
	// if (cell.address.indexOf("A") === 0) {
	// 	cell.font = {
	// 		bold: true
	// 	}
	// }
}

/**
 * 拷贝样式
 * @param {Object} style1
 * @param {Object} style2
 */
Excel.prototype.copy_style = function(style1, style2) {
	Object.assign(style1, JSON.parse(JSON.stringify(style2)));
}

/** 
 * 设置行样式
 * @param {Object} row 行
 * @param {Object} style 样式
 */
Excel.prototype.set_row_style = function(row, style, height = 0) {
	if (!style) {
		style = this.config.cell_style;
	}
	if (!height) {
		height = this.row_height;
	}
	if (height) {
		row.height = height;
	}
	var lt = this.row_style;
	if (lt) {
		var len = lt.length;
		row._cells.forEach((o) => {
			var se = {};
			var idx = o._column._number;
			if (idx <= len) {
				se = lt[idx - 1];
			} else {
				se = style;
			}
			this.copy_style(o.style, se);
			this.set_cell_func(o, se);
		});
	} else {
		row._cells.forEach((o) => {
			this.copy_style(o.style, style);
			this.set_cell_func(o, style);
		});
	}
}

/**
 * 获取操作表
 * @param {String|Number} name 名称
 */
Excel.prototype.get_sheet = function(name) {
	if (!this.book) {
		return this.book.getWorksheet(name || 1);
	} else if (name) {
		return this.sheet;
	}
}

/**
 * 保存表格前
 * @param {Object} sheet
 */
Excel.prototype.save_before = async function(sheet) {
	// 获取倒数第二行，并将其修改
	// var count = sheet.rowCount;
	// var row = sheet.getRow(count - 1);
	// var value = row.getCell(1).value;
	// console.log(value);
}

/**
 * 保存
 * @param {Array} jarr 列表
 * @param {Function} func 过滤回调函数
 * @param {String} file 文件路径
 * @return {String} 保存成功返回路径
 */
Excel.prototype.save = async function(jarr, func, file) {
	var f = file || this.config.file;
	var arr = f.split('.');
	f = f.fullname(this.config.path);
	var ext = arr[arr.length - 1];
	if (ext == 'xls') {
		ext = 'xlsx';
	}
	try {
		if (jarr) {
			var n = 1;
			var sheet_head;
			var tpl = this.config.tpl;
			if (tpl) {
				var sheet = await this.new_sheet();
				if (tpl.body) {
					sheet_body = await this.open(tpl.body);
					if (sheet_body) {
						var row = sheet_body.getRow(1);
						if (row) {
							this.row_height = row.height;
							this.row_style = row._cells.map((o) => {
								return o.style;
							});
						}
					}
				}
				sheet.columns = this.save_col();

				if (tpl.head) {
					sheet_head = await this.open(tpl.head);
					n = sheet_head.rowCount;
				}
				this.sheet = this.merge_sheet(sheet_head, sheet);
			}

			if (!this.sheet) {
				this.sheet = await this.new_sheet();
				this.sheet.columns = this.save_col();
			} else {
				this.sheet.columns = this.full_columns(this.sheet, n);
			}
			if (func) {
				for (var i = 0; i < jarr.length; i++) {
					var row = this.sheet.addRow(func(jarr[i]));
					this.set_row_style(row);
				}
			} else {
				for (var i = 0; i < jarr.length; i++) {
					var row = this.sheet.addRow(this.save_filter(jarr[i]));
					this.set_row_style(row);
				}
			}
			if (!sheet_head) {
				// 如果没有页头，则将第一行加粗
				var row = this.sheet.getRow(n);
				this.set_row_style(row);
				row.font = {
					bold: true
				};
			} else {
				this.sheet.spliceRows(n + 1, 1);
			}

			if (tpl && tpl.foot) {
				var sheet_foot = await this.open(tpl.foot);
				this.sheet = this.merge_sheet(this.sheet, sheet_foot);
			}
		} else if (this.sheet) {
			if (!this.book) {
				this.add_book();
			}
		} else {
			console.error('worksheet does not exist');
			return
		}

		var sheet;
		if (!this.book.worksheets.length) {
			sheet = this.book.addWorksheet(this.sheet.name);
		} else {
			sheet = this.book.getWorksheet(1);
		}
		Object.assign(sheet, this.sheet);

		var {
			pageSetup
		} = this.config;
		if (pageSetup) {
			Object.assign(sheet.pageSetup, pageSetup);
		}
		await this.save_before(this.sheet);
		await this.book[ext].writeFile(f, this.config.option);
	} catch (e) {
		console.error(e);
		f = '';
	}
	return f;
};

/**
 * 键值转换
 * @param {String} prop 属性
 * @param {String} key 键
 * @param {Object} value 值
 * @param {String} name = ['name'] 字段名, 用来取结果
 */
Excel.prototype.convert = function(prop, key, value, name = 'name') {
	var val;
	var list = this.format[prop];
	if (list) {
		for (var i = 0; i < list.length; i++) {
			var o = list[i];
			if (o[key] == value) {
				val = o[name]
				break;
			}
		}
	}
	return val;
};

/**
 * 新建一份表
 * @param {string} name 表名
 * @param {Object} tpl 引用模板 
 */
Excel.prototype.new_sheet = async function(name, tpl) {
	if (!this.book) {
		this.add_book();
	}
	return this.book.addWorksheet(name);
}

/**
 * 添加一行数据
 * @param {Object} data 需要插入的数据
 */
Excel.prototype.add_row = function(data, option) {
	return this.sheet.addRow(data, option);
}

/**
 * 查询指定行
 * @param {Number} row 需要查询的行
 */
Excel.prototype.get_row = function(row) {
	return this.sheet.getRow(row);
}

/**
 * 合并单元格
 * @param {String} posAB 单元格位置，例如 A1:B2
 * @param {String} value 合并后单元格的值
 */
Excel.prototype.merge = function(posAB, value) {
	var A = posAB.left(":");
	// 假设我们要合并的范围与已存在的合并范围冲突
	const cell = this.sheet.getCell(A);
	if (cell.isMerged) {
		this.sheet.unMergeCells(posAB); // 取消之前的合并
	}
	this.sheet.mergeCells(posAB);
	if (value) {
		this.sheet.getCell(A).value = value;
	}
}

/**
 * 通过数值获取列名
 * @param {Number} col 列值
 * @returns {String} 返回列值
 */
Excel.prototype.colname = function(col) {
	if (!col || col < 1) {
		col = 1;
	}
	var i = col - 1;
	var arr = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
		'U',
		'V', 'W', 'X', 'Y', 'Z', 'AA', 'AB', 'AC', 'AD', 'AE', 'AF', 'AG', 'AH', 'AI', 'AJ', 'AK', 'AL', 'AM',
		'AN', 'AO', 'AP',
		'Q', 'AR', 'AS', 'AT', 'AU', 'AV', 'AW', 'AX', 'AY', 'AZ'
	];
	return arr[i];
}

/**
 * 清理缓存
 */
Excel.prototype.clear = function() {
	this.list = [];
	this.original = [];
	this.book = null;
	this.sheet = null;
};

module.exports = Excel;