const Excel = require('./index.js');

var config = {
	file: './city.xls',
	convert: true,
	params: [{
			"name": "order_no",
			"title": "序号",
			"type": "number",
			"dataType": "mediumint"
		},
		{
			"name": "name",
			"title": "姓名\\日期",
			"type": "string",
			"dataType": "varchar"
		},
		{
			"name": "name",
			"title": "工时/天数",
			"type": "string",
			"dataType": "varchar"
		},
		{
			"name": "city_id",
			"title": "城市ID",
			"description": "",
			"type": "number",
			"dataType": "mediumint",
			"number": {
				"range": [
					1,
					8388607
				]
			}
		},
		{
			"name": "show",
			"title": "是否可见",
			"description": "0为仅表单可见，1为仅表单和搜索时可见 ，2为均可见",
			"type": "number",
			"dataType": "smallint",
			"number": {
				"max": 2
			}
		},
		{
			"name": "display",
			"title": "显示顺序",
			"description": "",
			"type": "number",
			"dataType": "smallint",
			"number": {
				"max": 1000
			}
		},
		{
			"name": "province_id",
			"title": "所属省份ID",
			"description": "",
			"type": "number",
			"dataType": "mediumint",
			"number": {
				"range": [
					1,
					8388607
				]
			}
		},
		{
			"name": "name",
			"title": "城市名称",
			"description": "",
			"type": "string",
			"dataType": "varchar",
			"string": {
				"notEmpty": 1
			}
		}
	],
	format: [{
			title: '所属省份',
			id: 'province_id',
			key: 'province',
			name: 'name',
			list: [{
					province_id: 1,
					name: '广东省'
				},
				{
					province_id: 2,
					name: '广西省'
				},
				{
					province_id: 3,
					name: '湖南省'
				}
			]
		},
		{
			title: '是否可见',
			key: 'show',
			list: ['否', '是']
		}
	]
};

/* 调用示例 - 读取xls表，并保存为xlsx表
 */
// async function test() {
// 	var body;
// 	var excel = new Excel(config);
// 	var jarr = await excel.load((o) => {
// 		if (o.show == '是' || o.show == '1' || o.show == 'Y') {
// 			o.show = 1;
// 		} else {
// 			o.show = 0;
// 		}
// 		return o;
// 	});

// 	console.log("读取到数据", jarr);
// 	var ex = new Excel(config);
// 	ex.config.file = 'city.xlsx';
// 	var file = await ex.save(jarr);
// 	console.log("保存为", file);
// 	ex.clear();
// }

// /**
//  * 创建一个带模板的新表，并保存为test.xlsx
//  */
// async function test() {
// 	config.tpl = {
// 		head: "./tpl/count_head.xlsx",
// 		body: "./tpl/count_body.xlsx",
// 		foot: "./tpl/count_foot.xlsx"
// 	}
// 	var excel = new Excel(config);
// 	var sheet_head = await excel.open(config.tpl.head);
// 	var sheet_body = await excel.open(config.tpl.body);
// 	var sheet_foot = await excel.open(config.tpl.foot);
// 	excel.clear();
// 	var sheet = excel.merge_sheet(sheet_head, sheet_body);
// 	excel.sheet = excel.merge_sheet(sheet, sheet_foot);
// 	excel.save(null, null, "./test.xlsx");
// 	// var jarr = [
// 	//   { city_id: 1, show: 1, display: 100, province_id: '广东省', name: '广州' },
// 	//   { city_id: 2, show: 0, display: 100, province_id: '广东省', name: '深圳' },
// 	//   { city_id: 3, show: 1, display: 100, province_id: '湖南省', name: '长沙' }
// 	// ];
// }


// /**
//  * 创建一个带模板的新表，并保存为test.xlsx
//  */
// async function test() {
// 	config.tpl = {
// 		head: "./tpl/count_head.xlsx",
// 		// body: "./tpl/count_body.xlsx",
// 		foot: "./tpl/count_foot.xlsx"
// 	}
// 	var excel = new Excel(config);
// 	var jarr = [{
// 			city_id: 1,
// 			show: 1,
// 			display: 100,
// 			province_id: '广东省',
// 			name: '广州',
// 			order_no: 1,
// 			date: "1970-01-01"
// 		},
// 		{
// 			city_id: 2,
// 			show: 0,
// 			display: 100,
// 			province_id: '广东省',
// 			name: '深圳',
// 			order_no: 2,
// 			date: "2024-03-31"
// 		},
// 		{
// 			city_id: 3,
// 			show: 1,
// 			display: 100,
// 			province_id: '湖南省',
// 			name: '长沙',
// 			order_no: 3,
// 			date: "2024-04-01"
// 		}
// 	];
// 	excel.save(jarr, null, "./test.xlsx");
// }

/**
 * 创建一个带模板的新表，并保存为test.xlsx
 */
async function test() {
	config.tpl = {
		// head: "./tpl/count_head.xlsx",
		// body: "./tpl/count_body.xlsx",
		foot: "./tpl/count_foot.xlsx"
	}
	var excel = new Excel(config);
	var jarr = "./data.json".loadJson();
	jarr.map((o)=> {
		// o[]
	});
	excel.save(jarr, null, "./test.xlsx");
}
// excel.add_row(["你好", "小白"], { at: 1 });
// excel.merge("A1:E1", "测试");
// excel.save();

test();